_G.ShowLeonWeaponPweaseUWU = ShowLeonWeaponPweaseUWU or {
	_mod_path = ModPath,
	_options_path = ModPath .. "options.txt",
	_save_path = SavePath .. "showweaponname_settings.txt",
	showweaponname_panel = "showweaponame_panel",
	_show_weapon_message_name = "switchweaponshowname",
	_playerstandard_hookid = "showweaponname_playerstandard_onequipweapon",
	_ps_updater_id = "update_attempt_register_playerstandard__start_action_equip_weapon",
	_pm_updater_id = "update_attempt_register_playermanager_checkskills",
	_hm_updater_id = "update_attempt_create_hud",
	_text_name = "weapon_name",
	_instance = ModInstance,
	_ws = nil, --just for name reference
	_panel = nil,
	_is_adjusting = false,
	font_size = 16,
	font = tweak_data.hud_players.ammo_font
}
ShowLeonWeaponPweaseUWU.settings = {
	enabled = true,
	hud_compatibility_mode = false,
	x = 0,
	y = 0,
	valign = 1,
	halign = 1,
	scale = 1,
	alpha = 1,
	hold_duration = 2,
	fadeout_duration = 1,
	allow_custom_weapon_names = true
}

function ShowLeonWeaponPweaseUWU.does_posthook_exist(class,id)
	if Hooks._function_hooks then 
		--SBLT versions 1.3.3 and later (as of 15 Dec 2021)
		if Hooks._function_hooks[class] and Hooks._function_hooks[class][id] then 
			return true
		end
	elseif Hooks._posthooks then 
		--legacy SBLT (v1.3.1 and earlier)
		for object_i, object in pairs( Hooks._posthooks ) do
			for func_i, func in pairs( object ) do
				for override_i, override in ipairs( func.overrides ) do
					if override.id == id then
						return true
					end
				end
			end
		end
	end
	
	return false
end

function ShowLeonWeaponPweaseUWU:IsEnabled()
	return self.settings.enabled
end

function ShowLeonWeaponPweaseUWU:GetPosition()
	return self.settings.x,self.settings.y
end

function ShowLeonWeaponPweaseUWU:GetHorizontalAlign()
	local halign = self.settings.halign
	if halign == 1 then 
		return "left"
	elseif halign == 2 then 
		return "center"
	elseif halign == 3 then 
		return "right"
	end
	return "left"
end

function ShowLeonWeaponPweaseUWU:GetVerticalAlign()
	local valign = self.settings.valign
	if valign == 1 then 
		return "top"
	elseif valign == 2 then 
		return "center"
	elseif valign == 3 then 
		return "bottom"
	end
	return "top"
end

function ShowLeonWeaponPweaseUWU:GetScale()
	return self.settings.scale
end

function ShowLeonWeaponPweaseUWU:GetAlpha()
	return self.settings.alpha
end

function ShowLeonWeaponPweaseUWU:GetWeaponName(slot)
	local player = managers.player and managers.player:local_player()
	if not alive(player) then 
		log("ShowWeaponName: ERROR: GetWeaponName(): Local player not found")
		return
	end
	local inventory = player:inventory()
	if not inventory then 
		log("ShowWeaponName: ERROR: GetWeaponName(): Inventory extension not found")
		return
	end
	
	local weapon
	if not slot then 
		slot = inventory:equipped_selection()
		weapon = inventory:equipped_unit()
	elseif type(slot) == "number" then 
		weapon = inventory:unit_by_selection(slot)
	else 
		slot = inventory:equipped_selection()
		if slot == 1 then 
			slot = 2
		elseif slot == 2 then
			slot = 1
		end
		weapon = inventory:unit_by_selection(slot)
	end
	
	local weaponbase = weapon and weapon:base()
	if weaponbase then
		local weapon_id = weaponbase.get_name_id and weaponbase:get_name_id() or weaponbase._name_id
		if weapon_id then 
			local weapon_name = managers.weapon_factory:get_weapon_name_by_weapon_id(weapon_id)
			if self:GetNicknamesAllowed() then
				if slot == 1 then 
					weapon_name = managers.blackmarket:equipped_secondary().custom_name or weapon_name
				elseif slot == 2 then 	
					weapon_name = managers.blackmarket:equipped_primary().custom_name or weapon_name
				end
			end
			return weapon_name
		end
	end
	
	log("ShowWeaponName: ERROR: GetWeaponName(" .. tostring(slot) .. "): No valid weapon found ")
	return "ERROR"
end

function ShowLeonWeaponPweaseUWU:GetNicknamesAllowed()
	return self.settings.allow_custom_weapon_names
end

function ShowLeonWeaponPweaseUWU:CreateHUD(hud,skip_layout)
	self._panel = hud:panel({
		name = "showweaponname_panel"
	})
	self._text = self._panel:text({
		name = self._text_name,
		text = "GEPGUN", 
		align = "left",
		vertical = "top",
		x = 0,
		alpha = 1, --panel alpha is managed on weapon switch, not text
		font = self.font,
		font_size = self.font_size,
		layer = 3
	})
	if not skip_layout then
		self:LayoutHUD()
	end
end

function ShowLeonWeaponPweaseUWU:LayoutHUD(set_adjusting,skip_stop_animate)
	if set_adjusting ~= nil then 
		self._is_adjusting = set_adjusting
	end
	local hud = self._panel
	if (hud and alive(hud)) then 
		if not skip_stop_animate then 
			hud:stop()
		end
		local text = hud:child("weapon_name")
		
		hud:set_visible(self:IsEnabled())
		
		local SCALE = self:GetScale()
		local HALIGN = self:GetHorizontalAlign()
		local VALIGN = self:GetVerticalAlign()

		text:set_vertical(VALIGN)
		text:set_align(HALIGN)
		text:set_alpha(1)
		hud:set_alpha(set_adjusting and 1 or 0) -- self:GetAlpha()
		local RAW_X,RAW_Y = self:GetPosition()

		text:set_font_size(self.font_size * SCALE)
		
		local x,y = 0,0
		if HALIGN == "left" then 
			x = RAW_X
		elseif HALIGN == "center" then
			x = RAW_X
		elseif HALIGN == "right" then 
			x = -RAW_X
		end
		if VALIGN == "top" then 
			y = RAW_Y
		elseif VALIGN == "center" then
			y = RAW_Y
		elseif VALIGN == "bottom" then 
			y = -RAW_Y
		end
		
		text:set_position(x,y)
		
	end
end

function ShowLeonWeaponPweaseUWU:CheckCompatibilityMode(enabled)
	local hm = managers.hud
	if not hm then 
		return false
	end
	if alive(self._panel) then 
		self._panel:stop()
		self._panel:parent():remove(self._panel)
	end
	
	if not (self._ws and alive(self._ws)) and managers.gui_data then 
		self._ws = managers.gui_data:create_fullscreen_workspace()
	end
	
	local parent_panel
	if enabled then 
		parent_panel = self._ws and self._ws:panel()
	else
		local hud = hm:script(PlayerBase.PLAYER_INFO_HUD_PD2)
		parent_panel = hud and hud.panel
	end
	
	if alive(parent_panel) then 
		self:CreateHUD(parent_panel)
	else
		log("ERROR: ShowWeaponName: No parent panel found!")
		return false
	end

	if not self.does_posthook_exist(HUDManager,self._playerstandard_hookid) then 
		local update_cb = callback(self,self,"UpdateRegisterWeaponSwitchHook")
		local updater_id = self._ps_updater_id
		if BeardLib then 
			BeardLib:RemoveUpdater(updater_id)
			BeardLib:AddUpdater(updater_id,update_cb)
		else
			hm:remove_updator(updater_id)
			hm:add_updator(updater_id,update_cb)
		end
	end
	
	return true
end

function ShowLeonWeaponPweaseUWU:UpdateRegisterWeaponSwitchHook(t,dt)
--only used to register the weapon switch message
	local pm = managers.player
	local player = pm and pm:local_player()
	
	Hooks:PostHook(PlayerStandard,"_start_action_equip_weapon",self._playerstandard_hookid,callback(self,self,"OnWeaponSwitch"))
	
	local updater_id = self._ps_updater_id
	if BeardLib then 
		BeardLib:RemoveUpdater(updater_id)
	elseif managers.hud then 
		managers.hud:remove_updator(updater_id)
	end
end

function ShowLeonWeaponPweaseUWU:UpdateCheckHUDCompatibilityMode(t,dt)
--only used to attempt making the correct hud each time
	if managers.hud and managers.hud:script(PlayerBase.PLAYER_INFO_HUD_PD2) then 
		if self:CheckCompatibilityMode(self.settings.hud_compatibility_mode) then 
			local updater_id = self._hm_updater_id
			if BeardLib then 
				BeardLib:RemoveUpdater(updater_id)
			else 
				managers.hud:remove_updator(updater_id)
			end
		end
	end
end

function ShowLeonWeaponPweaseUWU:OnWeaponSwitch()
	if self:IsEnabled() then 
		local panel = self._panel
		if not alive(panel) then 
			log("Show Weapon Name: ERROR: OnWeaponSwitch(): No panel found!")
			return
		end
		
		panel:stop()
		panel:set_alpha(self:GetAlpha())
		self._text:set_text(self:GetWeaponName())
		panel:animate(callback(self,self,"animate_fadeout"))
	end
end

function ShowLeonWeaponPweaseUWU:animate_fadeout(o)
	
	local elapsed = 0
	local dt
	local hold_duration = self.settings.hold_duration
	local fadeout_duration = self.settings.fadeout_duration
	local total_duration = hold_duration + fadeout_duration
	
--	local text = o:child(self._text_name):set_text(self:GetWeaponName(true))
	
	local function a()
		return string.format("%0.2f",Application:time())
	end
	
	repeat
		local _t = a()
		dt = coroutine.yield()
		
		if elapsed >= hold_duration then 
			--fadeout
			local progress = (elapsed - hold_duration) / fadeout_duration
			o:set_alpha(1 - progress)
--			Console:SetTrackerValue("trackerb","Fading " .. _t)
		else 
			--hold
--			Console:SetTrackerValue("trackera","Holding " .. _t)
		end
		
		elapsed = elapsed + dt
	until elapsed > total_duration 
	
--	Console:SetTrackerValue("trackerc","Done " .. a())
	o:set_alpha(0)
end

function ShowLeonWeaponPweaseUWU:Show()
	if self._panel and alive(self._panel) then 
		self._panel:set_alpha(1)
	end
end

function ShowLeonWeaponPweaseUWU:Hide()
	if self._panel and alive(self._panel) then 
		self._panel:set_alpha(0)
	end
end


Hooks:Add("LocalizationManagerPostInit", "ShowLeonWeaponPweaseUWU_LocalizationManagerPostInit", function( loc )
	if not BeardLib then 
		loc:load_localization_file(ShowLeonWeaponPweaseUWU._mod_path .. "loc/english.txt")
	end
end)

function ShowLeonWeaponPweaseUWU:Load()
	local file = io.open(self._save_path,"r")
	if file then
		for setting,value in pairs(json.decode(file:read("*all"))) do
			self.settings[setting] = value
		end
	else
		self:Save()
	end
end

function ShowLeonWeaponPweaseUWU:Save()
	local file = io.open(self._save_path,"w+")
	if file then
		file:write(json.encode(self.settings))
		file:close()
	end
end

Hooks:Add( "MenuManagerInitialize", "ShowLeonWeaponPweaseUWU_MenuManagerInitialize", function(menu_manager)

	MenuCallbackHandler.callback_showweaponname_set_enabled = function(self,item)
		local value = item:value() == "on"
		ShowLeonWeaponPweaseUWU.settings.enabled = value
		ShowLeonWeaponPweaseUWU:Save()
		ShowLeonWeaponPweaseUWU:LayoutHUD(true)
	end
	
	MenuCallbackHandler.callback_showweaponname_set_hud_compatibility_mode = function(self,item)
		local enabled = item:value() == "on"
		ShowLeonWeaponPweaseUWU.settings.hud_compatibility_mode = enabled
		ShowLeonWeaponPweaseUWU:CheckCompatibilityMode(enabled)
		ShowLeonWeaponPweaseUWU:Save()
	end
	
	MenuCallbackHandler.callback_showweaponname_set_hud_x = function(self,item)
		ShowLeonWeaponPweaseUWU.settings.x = tonumber(item:value())
		ShowLeonWeaponPweaseUWU:Save()
		ShowLeonWeaponPweaseUWU:LayoutHUD(true)
	end
	
	MenuCallbackHandler.callback_showweaponname_set_hud_y = function(self,item)
		ShowLeonWeaponPweaseUWU.settings.y = tonumber(item:value())
		ShowLeonWeaponPweaseUWU:Save()
		ShowLeonWeaponPweaseUWU:LayoutHUD(true)
	end
	
	MenuCallbackHandler.callback_showweaponname_set_hud_halign = function(self,item)
		ShowLeonWeaponPweaseUWU.settings.halign = tonumber(item:value())
		ShowLeonWeaponPweaseUWU:Save()
		ShowLeonWeaponPweaseUWU:LayoutHUD(true)
	end
	
	MenuCallbackHandler.callback_showweaponname_set_hud_valign = function(self,item)
		ShowLeonWeaponPweaseUWU.settings.valign = tonumber(item:value())
		ShowLeonWeaponPweaseUWU:Save()
		ShowLeonWeaponPweaseUWU:LayoutHUD(true)
	end
	
	MenuCallbackHandler.callback_showweaponname_set_scale = function(self,item)
		ShowLeonWeaponPweaseUWU.settings.scale = tonumber(item:value())
		ShowLeonWeaponPweaseUWU:Save()
		ShowLeonWeaponPweaseUWU:LayoutHUD(true)
	end
	MenuCallbackHandler.callback_showweaponname_set_fadeout_duration = function(self,item)
		ShowLeonWeaponPweaseUWU.settings.fadeout_duration = tonumber(item:value())
		ShowLeonWeaponPweaseUWU:Save()
	end
	MenuCallbackHandler.callback_showweaponname_set_hold_duration = function(self,item)
		ShowLeonWeaponPweaseUWU.settings.hold_duration = tonumber(item:value())
		ShowLeonWeaponPweaseUWU:Save()
	end
	MenuCallbackHandler.callback_showweaponname_set_alpha = function(self,item)
		ShowLeonWeaponPweaseUWU.settings.alpha = tonumber(item:value())
		ShowLeonWeaponPweaseUWU:Save()
		ShowLeonWeaponPweaseUWU:LayoutHUD(true)
	end
	
	MenuCallbackHandler.callback_showweaponname_set_allow_nicknames = function(self,item)
		ShowLeonWeaponPweaseUWU.settings.allow_custom_weapon_names = item:value() == "on"
		ShowLeonWeaponPweaseUWU:Save()
	end
	
	MenuCallbackHandler.callback_showweaponname_hititwithawrench = function(self)
		ShowLeonWeaponPweaseUWU:CheckCompatibilityMode(ShowLeonWeaponPweaseUWU.settings.hud_compatibility_mode)
		local title,desc
		if managers.player then
			title = "showweaponname_menu_reboot_1_title"
			desc = "showweaponname_menu_reboot_1_desc"
		else
			title = "showweaponname_menu_reboot_2_title"
			desc = "showweaponname_menu_reboot_2_desc"
		end
		
		QuickMenu:new(managers.localization:text(title),managers.localization:text(desc),{
			{
				text = ":)",
				is_cancel_button = true,
				is_default_button = true
			}
		},true)
	end
	
	MenuCallbackHandler.callback_showweaponname_close = function(self,item)
		ShowLeonWeaponPweaseUWU._is_adjusting = false
		ShowLeonWeaponPweaseUWU:Hide()
	end
	
	ShowLeonWeaponPweaseUWU:Load()
	
	MenuHelper:LoadFromJsonFile(ShowLeonWeaponPweaseUWU._options_path, ShowLeonWeaponPweaseUWU, ShowLeonWeaponPweaseUWU.settings)
end)


--this is only used for hud compatibility
--when changing the option while already in-game, in order for the changes to take effect, the user must either:
--1. restart the game, or
--2. change the HUD compatibility option in the in-game menu, or
--3. use the "hit it with a wrench" button in the in-game menu
Hooks:Add("BaseNetworkSessionOnLoadComplete","ShowLeonWeaponPweaseUWU_OnLoadComplete",function() 
	if ShowLeonWeaponPweaseUWU.settings.hud_compatibility_mode then 
		local updater_id = ShowLeonWeaponPweaseUWU._hm_updater_id
		local update_cb = callback(ShowLeonWeaponPweaseUWU,ShowLeonWeaponPweaseUWU,"UpdateCheckHUDCompatibilityMode")
		if BeardLib then 
			BeardLib:RemoveUpdater(updater_id)
			BeardLib:AddUpdater(updater_id,update_cb)
		elseif managers.hud then
			managers.hud:remove_updator(updater_id)
			managers.hud:add_updator(updater_id,update_cb)
		else
			log("ShowWeaponName: Attempted to check compatibility mode, but BeardLib is not installed, and managers.hud is not initialized yet. Guess I'll die.")
		end
	end
end)

